'''
Scraping openphish
'''
from selenium import webdriver
import pandas as pd
from selenium.common.exceptions import NoSuchElementException, TimeoutException
import tldextract
from urllib.parse import urlparse
import time
import random
from bs4 import BeautifulSoup
import sys
import os
import math
import glob

USER_AGENTS = [
"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36",
"Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36",
"Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36",
"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36",
"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36",
"Mozilla/5.0 (iPhone; CPU iPhone OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/83.0.4147.71 Mobile/15E148 Safari/604.1",
"Mozilla/5.0 (iPad; CPU OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/83.0.4147.71 Mobile/15E148 Safari/604.1",
"Mozilla/5.0 (iPod; CPU iPhone OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/83.0.4147.71 Mobile/15E148 Safari/604.1",
"Mozilla/5.0 (Linux; Android 10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Mobile Safari/537.36",
"Mozilla/5.0 (Linux; Android 10; SM-A205U) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Mobile Safari/537.36"
]

def get_2ld(fqdn, only_tld = False):
    try:
        ext = tldextract.extract(fqdn)
        if len(ext.domain) == 0 or len(ext.suffix) == 0:
            return None
        else:
            if only_tld:
                return ext.suffix
            else: 
                return ext.domain + "." + ext.suffix
    except Exception as e:
        pass
    return None

def get_fqdn(url):
    try:
        uri = urlparse(url)
        if uri != None:
            return uri.netloc
    except Exception as e:
        pass

def start_browser(agent_index):
    #brew cask install chromedriver
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument("--incognito")
    chrome_options.add_argument("--headless")
    chrome_options.add_argument('user-agent="{}"'.format(USER_AGENTS[agent_index]))
    browser = webdriver.Chrome(executable_path=r"chromedriver", options=chrome_options)

    browser.set_page_load_timeout(10)

    return browser

def fetch_url(browser, url):
    try:
        browser.get(url)
        return browser.page_source
    except Exception as e:
        return "<html>Error: {}</html>".format(e) 

def close_browser(browser):
    browser.close()

def save_links(response, outfile):
    f1 = open(outfile, "w")
    f1.write(response)
    f1.close()

def parse_page(outfile):
    f2 = open(outfile + "_parsed", "a")
    soup = BeautifulSoup(open(outfile).read(), features = "html.parser")
    table = soup.find("table", {"class": "pure-table pure-table-striped"})
    if table != None:
        tbody = table.find("tbody")
        if tbody != None:
            trs = tbody.find_all("tr")
            for tr in trs:
                tds = tr.find_all("td")
                row = []
                for td in tds:
                    row.append(td.text)
                f2.write("{}\n".format("#####".join(row)))
    f2.close()

def save_page(outfile):
    agent_index = math.floor(random.uniform(0, len(USER_AGENTS)))
    browser = start_browser(agent_index)
    try:
        url = "https://openphish.com/"
        save_links(fetch_url(browser, url), outfile + "_raw")
        parse_page(outfile + "_raw")
    except Exception as e:
        print(e)
        pass
    close_browser(browser)

if __name__ == "__main__":
    option = sys.argv[1]
    if option == "page":
        #generic file prefix
        save_page(sys.argv[2])



import psycopg2
import sys
from datetime import datetime
import datetime 

user_name = "vtuser"
paswd = "vtuser"
host_name = "10.4.8.73"
port_id = "5432"
database_name = "vt_snapshot"

def get_conn():
    conn = psycopg2.connect(user = user_name,
                            password = paswd,
                            host = host_name,
                            port = port_id,
                            database = database_name)
    return conn

def get_vt_status(db, domain):
    if domain == None or len(domain) == 0:
        print("WARNING: Empty domain")
        return None

    c = db.cursor()
    params = (domain, )
    query = "SELECT domain, first_detected, last_detected, malcount, suscount FROM vt WHERE domain = %s"
    c.execute(query, params)
    record = c.fetchone()
    if record != None:
        record = list(record)
        record[1] = record[1].strftime("%Y%m%d")
        record[2] = record[2].strftime("%Y%m%d")
    c.close()
    return record 

def get_vt_status_file(db, infile, outfile):
    of = open(outfile, "w")
    inf = open(infile)

    for domain in inf:
        domain = domain.strip()
        res = get_vt_status(db, domain)
        if res == None:
            of.write("{} {}\n".format(domain, "##NOTFOUND##"))
        else:
            res2 = [str(s) for s in res]
            of.write("{}\n".format(" ".join(res2)))
    of.close()

if __name__ == "__main__":
    db = get_conn()
    if db != None:
        if sys.argv[1] == "domain_lookup":
            print(get_vt_status(db, sys.argv[2]))
        elif sys.argv[1] == "file_lookup":
            get_vt_status_file(db, sys.argv[2], sys.argv[3])
        else:
            print("Unsupported option")
        db.close()



#!/usr/bin/python
'''
Author: @nabeelxy
Description:
This file is specifically prepared for phicious project
Given a list of domains in a file, it produces a file with the following features:
'''

import sys
import pdns
from datetime import datetime
import tldextract

def get_2ld(fqdn):
    ext = tldextract.extract(fqdn)
    return ext.domain + "." + ext.suffix

def get_ip_features(ip, firstseen_before = None):
    first_seen = None
    last_seen = None
    count = 0
    domain_set = set()
    first_seen_date = None
    last_seen_date = None
    LAST_SEEN_AFTER = "20200601 00:00"
    mappings = pdns.pdns_ip(ip, firstseen_before, None, None, LAST_SEEN_AFTER, 5000)

    if mappings == None:
        return None
    
    for entry in mappings:
        if first_seen == None:
            first_seen = entry["time_first"]
        elif first_seen > entry["time_first"]:
            first_seen = entry["time_first"]
        if last_seen == None:
            last_seen = entry["time_last"]
        elif last_seen < entry["time_last"]:
            last_seen = entry["time_last"]
        domain_set.add(get_2ld(entry["rrname"]))
        count += entry["count"]
    #firstseen, lastseen, query count, #apex_domains
    return [first_seen, last_seen, count, len(domain_set)]

def get_features_file(domain_file, outfile, firstseen_before = None):
    df = open(domain_file, encoding = 'utf-8')
    of = open(outfile, "w+", encoding = 'utf-8')
    for ip in df:
        ip = ip.strip()
        out = get_ip_features(ip, firstseen_before)
        if out != None:
            first_seen_date = None
            last_seen_date = None
            if out[0] != None:
                first_seen_date = datetime.fromtimestamp(out[0])
            if out[1] != None:
                last_seen_date = datetime.fromtimestamp(out[1])
            of.write("{},{},{},{},{}\n".format(ip, first_seen_date, last_seen_date, out[2], out[3]))
    of.close()  
    df.close() 

if __name__ == "__main__":
    if len(sys.argv) < 4:
        print("usage: python {} input_domain_file output_file firstseen_before(YYYYMMDD HH:MM)".format(sys.argv[0]))
        exit()
        
    domainf = sys.argv[1] #input domain  file
    outf = sys.argv[2] #output file
    firstseen_before = sys.argv[3]
    if firstseen_before.lower() == "none":
        firstseen_before = None
    get_features_file(domainf, outf, firstseen_before)



import pdns
import sys


def save_all(resource, fh, limit, server_ip, resource_type):
    entries = list()
    if resource_type == "domain":
        entries = pdns.pdns_domain(resource, None, None, None, None, limit, "A", server_ip)
        print(entries)
    elif resource_type == "ip":
        entries = pdns.pdns_ip(resource, None, None, None, None, limit, server_ip)

    if entries != None:
        for entry in entries:
            fh.write("{}\n".format(entry))

def all_resources(resourcef, outfile, limit, collected, server_ip, resource_type):
    done = set()
    with open(collected) as c:
        for d in c:
            done.add(d.strip())

    fh = open(outfile, "w", 512)
    df = open(resourcef)
    for resource in df:
        try:
            if resource_type == "domain" or resource_type == "ns":
                resource = resource.strip().strip(".").encode("idna").decode()
            else:
                resource = resource.strip()
            if resource in done:
                continue
            save_all(resource, fh, limit, server_ip, resource_type)
        except Exception as e:
            pass
    fh.close()

if __name__ == "__main__":
    if len(sys.argv) < 6:
        print("Usage: resource_file output_file limit collected_file server_ip resource_type (domain|ip|ns)")
        exit()

    resourcefile = sys.argv[1]
    outfile = sys.argv[2]
    limit = int(sys.argv[3])
    collected = sys.argv[4]
    server_ip = sys.argv[5]
    resource_type = sys.argv[6] #domain|ip|ns

    if resource_type not in {"domain", "ip", "ns"}:
        print("Invalid resource type: {}".format(resource_type))
        exit()

    all_resources(resourcefile, outfile, limit, collected, server_ip, resource_type)

    



'''
Domain/IP/AS lookup using RDAP service
Reference: https://about.rdap.org/
Rate limit: below 10 queries per second
'''
import requests
import sys
import time

#type (domain, ip, autnum)
ENDPOINT = 'https://rdap.org/{}/{}'

#rdap lookup
def rdap_get(rtype, value):
    try:
        req = requests.get(ENDPOINT.format(rtype, value))
        if req.status_code == 200:
            output = req.json()
        elif req.status_code == 204:
            output = {'ERROR': 'Received HTTP 204: You may have exceeded the rate limit', 'url':url}
    except requests.ConnectTimeout as e:
        output = {'ERROR': '##timed out', 'url':url}
    return output

#file lookup
def rdap_get_file(rtype, inputf, outputf, interval = 0.15):
    inf = open(inputf)
    outf = open(outputf, "w")
    for line in inf:
        outf.write("{}\n".format(rdap_get(rtype, line.strip())))
        time.sleep(interval)
    outf.close()
        
if __name__ == "__main__":
    #print(rdap_get(sys.argv[1], sys.argv[2]))
    if len(sys.argv) < 4:
        print("Usage: {} rtype inputfile outputfile".format(sys.argv[0]))
        exit()
    rdap_get_file(sys.argv[1], sys.argv[2], sys.argv[3])



'''
Description: Parsing VT reports
Author: nabeelxy
'''
import glob
import json
import sys
import tarfile
from datetime import datetime, timedelta
from io import IOBase

import pandas as pd

BASE_VT_DIR = "/export/sec03/vt/vt_url_feed"


def load_data(in_file):
    df = pd.read_csv(in_file, header=None)
    df.columns = ['name']
    return set(df['name'].unique())


def update_data(in_file, new_data):
    all_data = new_data
    try:
        df = pd.read_csv(in_file, header=None)
        df.columns = ['name']
        all_data = set(df['name'].unique()) | new_data
    except:  # if the file is empty or non-existent
        pass
    pd.DataFrame(all_data).to_csv(in_file, header=False, index=False)


# From all VT reports find out all attack types and engine names
# rf - file handle
def get_attacks_engines(rf):
    attacks = set()
    engines = set()

    for line in rf:
        try:
            jobj = json.loads(line.strip())
            scans = jobj["scans"]
            for key, value in scans.items():
                engines.add(key)
                attacks.add(value['result'])
        except KeyError as e:
            pass
    if isinstance(rf, IOBase):
        rf.seek(0)

    return attacks, engines


def vt2df(rf, attacks=None, engine_names=None, only_pos=True, attack_labels=False):
    # if attacks and engien names are not specified, first get the sets
    if not attacks and not engine_names:
        attacks, engine_names = get_attacks_engines(rf)

    df = pd.DataFrame()

    records = list()

    for line in rf:
        try:
            jobj = json.loads(line.strip())
            record = dict()
            record["positives"] = jobj["positives"]
            if only_pos and record["positives"] == 0:
                continue
            url = jobj["url"].strip("/")
            record["url"] = url
            record["scan_date"] = jobj["scan_date"]
            record["scan_id"] = jobj["scan_id"]
            record["total"] = jobj["total"]
            record["first_seen"] = jobj["first_seen"]
            record["last_seen"] = jobj["last_seen"]

            # additional information
            record["sophos category"] = None
            try:
                record["sophos category"] = jobj["additional_info"]["sophos category"]
            except:
                pass
            record["Webroot category"] = None
            try:
                record["Webroot category"] = jobj["additional_info"]["Webroot category"]
            except:
                pass
            record["x-xss-protection"] = None
            try:
                record["x-xss-protection"] = jobj["additional_info"]["Response headers"]["x-xss-protection"]
            except:
                pass
            record["x-frame-options"] = None
            try:
                record["x-frame-options"] = jobj["additional_info"]["Response headers"]["x-frame-options"]
            except:
                pass
            record["set-cookie"] = None
            try:
                record["set-cookie"] = jobj["additional_info"]["Response headers"]["set-cookie"]
            except:
                pass
            record["Response content SHA-256"] = None
            try:
                record["Response content SHA-256"] = jobj["additional_info"]["Response content SHA-256"]
            except:
                pass
            record["Response code"] = None
            try:
                record["Response code"] = jobj["additional_info"]["Response code"]
            except:
                pass
            record["Forcepoint ThreatSeeker category"] = None
            try:
                record["Forcepoint ThreatSeeker category"] = jobj["additional_info"]["Forcepoint ThreatSeeker category"]
            except:
                pass

            record["Comodo Valkyrie Verdict category"] = None
            try:
                record["Comodo Valkyrie Verdict category"] = jobj["additional_info"]["Comodo Valkyrie Verdict category"]
            except:
                pass

            record["ip"] = None
            try:
                record["ip"] = jobj["additional_info"]["resolution"]
            except:
                pass
            record["content_hash"] = None
            try:
                record["content_hash"] = jobj["additional_info"]["Response content SHA-256"]
            except:
                pass
            record["rlength"] = None
            try:
                record["rlength"] = jobj["additional_info"]["rlength"]
            except:
                pass
            record["url_after_redirects"] = None
            try:
                record["url_after_redirects"] = jobj["additional_info"]["URL after redirects"]
            except:
                pass

            for attack in attacks:
                record[attack] = 0
            for engine in engine_names:
                record[engine] = 1

            # 1 - not present
            # 2 - unrated site
            # 3 - marked benign
            # 4 - marked malicious
            scans = jobj["scans"]
            if attack_labels:  # mark with attack label
                for key, value in scans.items():
                    record[value['result']] += 1
                    record[key] = value['result']
            else:  # use the above code for detected/undtected sites
                for key, value in scans.items():
                    record[value['result']] += 1
                    if value['detected']:
                        record[key] = 4  # marked malicious
                    elif value['result'] == "unrated site":
                        record[key] = 2
                    else:
                        record[key] = 3  # marked bengin

            records.append(record)
        except KeyError as e:
            print(e)
            pass
    return pd.DataFrame(records)


def vt_parse_attacks_engines_dir(startdate, enddate, basedir=BASE_VT_DIR):
    s_date = datetime.strptime(startdate.strip(), "%Y%m%d").date()
    e_date = datetime.strptime(enddate.strip(), "%Y%m%d").date()
    delta = e_date - s_date
    files = []
    for i in range(delta.days + 1):
        curr_date = s_date + timedelta(i)
        files += glob.glob("{}/url-feed-{}T*.tar.bz2".format(basedir, curr_date.strftime("%Y%m%d")))

    return vt_parse_attacks_engines(files)


# find the unique attacks and engines mentioned in the report
# VT report engines change over time - hence we need to reprofile
def vt_parse_attacks_engines(files):
    master_attacks = set()
    master_engines = set()
    for fi in files:
        try:
            if fi.endswith(".bz2"):
                tar = tarfile.open(fi, "r:bz2")
                for item in tar:
                    fh = tar.extractfile(item)
                    attacks, engines = get_attacks_engines(fh)
                    master_attacks |= attacks
                    master_engines |= engines
            else:  # uncompressed files
                fh = open(fi)
                attacks, engines = get_attacks_engines(fh)
                master_attacks |= attacks
                master_engines |= engines
        except Exception as e:
            print(e)
            pass
            print("ERROR: parsing file {}".format(fi))

    return master_attacks, master_engines


def vt_parse_data_dir(startdate, enddate, attacks, engines, only_pos=True, basedir=BASE_VT_DIR, attack_labels=False):
    s_date = datetime.strptime(startdate.strip(), "%Y%m%d").date()
    e_date = datetime.strptime(enddate.strip(), "%Y%m%d").date()
    delta = e_date - s_date
    files = []
    for i in range(delta.days + 1):
        curr_date = s_date + timedelta(i)
    files += glob.glob("{}/url-feed-{}T*.tar.bz2".format(basedir, curr_date.strftime("%Y%m%d")))
    return vt_parse_data(startdate, enddate, attacks, engines, only_pos, files, attack_labels)


def vt_parse_data_files(attacks, engines, only_pos, files, attack_labels):
    if attacks == None and engines == None:
        attacks, engines = vt_parse_attacks_engines(startdate, enddate)

    dfs = list()
    df = None
    for fi in files:
        try:
            if fi.endswith(".bz2"):
                tar = tarfile.open(fi, "r:bz2")
                for item in tar:
                    fh = tar.extractfile(item)
                    df = vt2df(fh, attacks, engines, only_pos, attack_labels)
                    # de-dup records
                    # current policy - keep the record with the highest positive (we may change this later)
                    df = df.sort_values(by="positives", ascending=False)
                    df = df.drop_duplicates(subset=["url"], keep="first")
                    dfs.append(df)
            else:  # regular file
                fh = open(fi)
                df = vt2df(fh, attacks, engines, only_pos, attack_labels)
                # de-dup records
                # current policy - keep the record with the highest positive (we may change this later)
                df = df.sort_values(by="positives", ascending=False)
                df = df.drop_duplicates(subset=["url"], keep="first")
                dfs.append(df)
        except Exception as e:
            print(e)
            pass
            print("ERROR: parsing file {}".format(fi))
    if len(dfs) > 0:
        df = pd.concat(dfs)
        df = df.sort_values(by="positives", ascending=False)
        df = df.drop_duplicates(subset=["url"], keep="first")

    return df


if __name__ == "__main__":
    option = sys.argv[1]  # engines|engines_file|parse|parse_file

    if option == "engines":  # parse a set of files in a folder that are annotated with date
        startdate = sys.argv[2]  # YYYYMMDD
        enddate = sys.argv[3]  # YYYYMMDD
        basedir = sys.argv[4]
        attacks, engines = vt_parse_attacks_engines_dir(startdate, enddate, basedir)
        update_data("attacks.csv", attacks)
        update_data("engines.csv", engines)
    elif option == "engines_file":
        infile = sys.argv[2]  # input file
        attacks, engines = vt_parse_attacks_engines([infile])
        update_data("attacks.csv", attacks)
        update_data("engines.csv", engines)
    elif option == "parse":  # parse a set of files in a folder that are annotated with date
        startdate = sys.argv[2]  # YYYYMMDD
        enddate = sys.argv[3]  # YYYYMMDD
        basedir = sys.argv[4]
        outfile = sys.argv[5]  # file where we are going to save the parsed data
        labels = int(sys.argv[5])  # 1 for true, 0 for false
        attacks = load_data("attacks.csv")
        engines = load_data("engines.csv")
        df = vt_parse_data_dir(startdate, enddate, attacks, engines, True, basedir, labels)
        df.to_csv(outfile, header=True, index=False)
    elif option == "parse_file":  # parse a single file
        infile = sys.argv[2]
        outfile = sys.argv[3]
        labels = int(sys.argv[4])  # 1 for true, 0 for false
        attacks = load_data("attacks.csv")
        engines = load_data("engines.csv")
        df = vt_parse_data_files(attacks, engines, False, [infile], labels)
        df.to_csv(outfile, header=True, index=False)
    else:
        print("Unrecognized engine")
